
import requests
from time import time
from io import StringIO
from rkd.api.inputoutput import IO
from rkd_harbor.test import BaseHarborTestClass
from rkd_harbor.service import ServiceDeclaration
from rkd_harbor.exception import ServiceNotCreatedException
from rkd_harbor.exception import ServiceNotReadyException


class ComposeDriverTest(BaseHarborTestClass):
    def test_get_compose_args_sets_basic_arguments_and_sees_yaml_configuration_files(self):
        drv = self._get_prepared_compose_driver()
        args = drv.get_compose_args()

        self.assertIn('--project-directory=', args, msg='Project directory should be set')
        self.assertIn('-p env_simple', args, msg='Project name should be set')
        self.assertIn('-f docker-compose.yml', args, msg='Basic docker-compose.yml should be considered')
        self.assertIn('infrastructure.ssl.yml', args)
        self.assertIn('infrastructure.service-discovery.yml', args)
        self.assertIn('infrastructure.health.yml', args)

    def test_ps(self):
        """Simply check if docker-compose ps can be executed, if the standard switches are correct"""

        io_str = StringIO()
        io = IO()

        with io.capture_descriptors(stream=io_str, enable_standard_out=False):
            drv = self._get_prepared_compose_driver()
            drv.ps([])

        self.assertIn('Name', io_str.getvalue())
        self.assertIn('Ports', io_str.getvalue())

    def test_up(self):
        """Simply verify that the docker-compose up command is properly generated"""

        resulting_args = []

        # mock
        drv = self._get_prepared_compose_driver()
        drv.scope.sh = lambda *args, **kwargs: (resulting_args.append(args))

        # action
        drv.up(ServiceDeclaration('healthcheck', {}))

        args_as_str = ' '.join(resulting_args[0])

        self.assertIn('--scale healthcheck=1 healthcheck', args_as_str)
        self.assertIn('infrastructure.health.yml', args_as_str)
        self.assertIn('up -d', args_as_str)

    def test_up_invalid_parameters_for_recreation(self):
        """Unit test: Checks if exception is raised when logic error happens"""

        drv = self._get_prepared_compose_driver()
        drv.scope.sh = lambda *args, **kwargs: ''

        # action
        self.assertRaises(Exception, lambda: drv.up(ServiceDeclaration('healthcheck', {}),
                                                    norecreate=True, force_recreate=True))

    def test_up_tolerates_extra_arguments(self):
        """Unit tests: Checks if extra arguments are allowed to be passed"""

        resulting_args = []
        drv = self._get_prepared_compose_driver()
        drv.scope.sh = lambda *args, **kwargs: (resulting_args.append(args))

        # action
        drv.up(ServiceDeclaration('healthcheck', {}), extra_args='--remove-orphans')
        args_as_str = ' '.join(resulting_args[0])

        self.assertIn('--remove-orphans', args_as_str)

    def test_functional_up_and_down(self):
        """Functional test to start, stop, restart, remove container
           Checks basic compatibility with docker-compose interface
        """

        drv = self._get_prepared_compose_driver()

        # 1) Start
        drv.up(ServiceDeclaration('gateway', {}))
        self.assertTrue(self.get_containers_state(drv)['env_simple_gateway_1'])

        # 2) Restart
        drv.restart('gateway')
        self.assertTrue(self.get_containers_state(drv)['env_simple_gateway_1'])

        # 3) Stop
        drv.stop('gateway')
        self.assertFalse(self.get_containers_state(drv)['env_simple_gateway_1'])

        # 4) Remove
        drv.rm(ServiceDeclaration('gateway', {}))
        self.assertNotIn('env_simple_gateway_1', self.get_containers_state(drv))

    def test_get_last_container_name_for_service(self):
        drv = self._get_prepared_compose_driver()

        # make sure it is clean up
        drv.rm(ServiceDeclaration('gateway', {}), capture=True)  # there may be more instances, so remove them
        drv.up(ServiceDeclaration('gateway', {}), capture=True)  # bring one instance

        self.assertEqual('env_simple_gateway_1', drv.get_last_container_name_for_service('gateway'))

    def test_find_container_name_case_service_not_created_raises_exception(self):
        drv = self._get_prepared_compose_driver()

        self.assertRaises(
            ServiceNotCreatedException,
            lambda: drv.find_container_name(ServiceDeclaration('not_existing', {}))
        )

    def test_find_container_name_finds_first_instance_when_no_instance_specified(self):
        drv = self.prepare_example_service('gateway')

        instance_name = drv.find_container_name(ServiceDeclaration('gateway', {}))
        self.assertEqual('env_simple_gateway_1', instance_name)

    def test_find_container_name_finds_first_instance_by_specyfing_instance_num(self):
        drv = self.prepare_example_service('gateway')

        self.assertEqual(
            'env_simple_gateway_1',
            drv.find_container_name(ServiceDeclaration('gateway', {}), instance_num=1)
        )

    def test_find_container_name_does_not_find_instance_when_there_is_not_enough_replicas(self):
        drv = self.prepare_example_service('gateway')

        # for single works
        self.assertEqual('env_simple_gateway_1', drv.find_container_name(ServiceDeclaration('gateway', {}), instance_num=1))

        self.assertRaises(
            ServiceNotCreatedException,
            lambda: drv.find_container_name(ServiceDeclaration('gateway', {}), instance_num=50)
        )

    def test_wait_for_log_message_finds_searched_phrase(self):
        drv = self.prepare_example_service('gateway')

        requests.get('http://localhost:8000')
        self.assertTrue(
            drv.wait_for_log_message('GET / HTTP/1.1', ServiceDeclaration('gateway', {}), timeout=5)
        )

    def test_wait_for_log_hits_timeout(self):
        drv = self.prepare_example_service('gateway')

        start_time = time()

        self.assertRaises(
            ServiceNotReadyException,
            lambda: drv.wait_for_log_message(
                'Some message that should be there',
                ServiceDeclaration('gateway', {}),
                timeout=2
            )
        )

        self.assertGreaterEqual(time() - start_time, 2, msg='Expected at least two seconds waiting')

    def test_scale_one_up_then_scale_down_to_desired_state(self):
        """Functional test: Scale one up, then down to declared state

        Covers:
            scale_one_up()
            scale_to_desired_state()
        """

        drv = self.prepare_example_service('website')
        drv.scale_one_up(ServiceDeclaration('website', {}))

        running_containers = self.get_containers_state(drv)
        self.assertIn('env_simple_website_1', running_containers)
        self.assertIn('env_simple_website_2', running_containers)

    def test_find_all_container_names_for_service_finds_containers(self):
        drv = self.prepare_example_service('website')

        self.assertEqual(['env_simple_website_1'],
                         drv.find_all_container_names_for_service(ServiceDeclaration('website', {})))

        drv.scale_one_up(ServiceDeclaration('website', {}))
        containers = drv.find_all_container_names_for_service(ServiceDeclaration('website', {}))

        self.assertIn('env_simple_website_1', containers)
        self.assertIn('env_simple_website_2', containers)

    def test_find_all_container_names_for_service_raises_exception_on_invalid_service_name(self):
        drv = self._get_prepared_compose_driver()

        self.assertRaises(
            ServiceNotCreatedException,
            lambda: drv.find_all_container_names_for_service(ServiceDeclaration('not_existing', {}))
        )

    def test_get_created_containers_is_returning_containers_in_proper_order(self):
        """Test that get_created_containers() is preserving order"""

        drv = self._get_prepared_compose_driver()

        # create three instances of a service
        drv.up(ServiceDeclaration('website', {}))
        drv.scale_one_up(ServiceDeclaration('website', {}))

        containers = drv.get_created_containers(only_running=False)
        self.assertEqual([1, 2], list(containers['website'].keys()))

    def test_get_created_containers_is_showing_only_running_containers(self):
        """Test that 'website' service will be created, but will have no active containers"""

        drv = self._get_prepared_compose_driver()

        # create three instances of a service
        drv.up(ServiceDeclaration('website', {}))
        drv.stop('website')

        containers = drv.get_created_containers(only_running=True)
        self.assertEqual([], list(containers['website']))



from typing import Dict
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from ...formatting import development_formatting
from ...exception import MissingDeploymentConfigurationError
from .base import BaseDeploymentTask


class UpdateFilesTask(BaseDeploymentTask):
    """Updates an Ansible role and required configuration files.
    Warning: Overwrites existing files, but does not remove custom files in '.rkd/deployment' directory"""

    def get_name(self) -> str:
        return ':update'

    def get_group_name(self) -> str:
        return ':harbor:deployment:files'

    def format_task_name(self, name) -> str:
        return development_formatting(name)

    def get_declared_envs(self) -> Dict[str, str]:
        envs = super(BaseDeploymentTask, self).get_declared_envs()
        envs['VAULT_PASSWORDS'] = ''

        return envs

    def configure_argparse(self, parser: ArgumentParser):
        self._add_ask_pass_arguments_to_argparse(parser)
        self._add_vault_arguments_to_argparse(parser)

    def run(self, context: ExecutionContext) -> bool:
        self._preserve_vault_parameters_for_usage_in_inner_tasks(context)

        try:
            return self.install_and_configure_role(context, force_update=True)

        except MissingDeploymentConfigurationError as e:
            self.io().error_msg(str(e))
            return False



import os
import subprocess
from typing import Dict
from typing import Tuple
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from ..base import HarborBaseTask
from ...formatting import development_formatting
from ...exception import MissingDeploymentConfigurationError
from .base import BaseDeploymentTask

HARBOR_ROOT = os.path.dirname(os.path.realpath(__file__)) + '/../../deployment/files'


class DeploymentTask(BaseDeploymentTask):
    """Deploys your project from GIT to a PRODUCTION server

All changes needs to be COMMITTED and PUSHED to GIT server, the task does not copy local files.

The deployment task can be extended by environment variables and switches to make possible any customizations
such as custom playbook, custom role or a custom inventory. The environment variables from .env are considered.

Example usage:
    # deploy services matching profile "gateway", use password stored in .vault-password for Ansible Vault
    harbor :deployment:apply -V .vault-password --profile=gateway

    # another example with Vault, multiple passwords, and environment variable usage
    # (NOTICE: paths to password files must begin with "/" or "./")
    VAULT_PASSWORDS="./.vault-password-file||other-plain-text-password" harbor :deployment:apply

    # deploy from different branch
    harbor :deployment:apply --branch production_fix_1

    # use SSH-AGENT & key-based authentication by specifying path to private key
    harbor :deployment:apply --git-key=~/.ssh/id_rsa
    """

    def get_name(self) -> str:
        return ':apply'

    def get_group_name(self) -> str:
        return ':harbor:deployment'

    def format_task_name(self, name) -> str:
        return development_formatting(name)

    def get_declared_envs(self) -> Dict[str, str]:
        envs = super(DeploymentTask, self).get_declared_envs()
        envs['PLAYBOOK'] = 'harbor.playbook.yml'
        envs['INVENTORY'] = 'harbor.inventory.cfg'
        envs['GIT_KEY'] = ''
        envs['VAULT_PASSWORDS'] = ''

        return envs

    def configure_argparse(self, parser: ArgumentParser):
        parser.add_argument('--playbook', '-p', help='Playbook name', default='harbor.playbook.yml')
        parser.add_argument('--git-key', '-k', help='Path to private key for a git repository eg. ~/.ssh/id_rsa',
                            default='')
        parser.add_argument('--inventory', '-i', help='Inventory filename', default='harbor.inventory.cfg')
        parser.add_argument('--debug', '-d', action='store_true', help='Set increased logging for Ansible output')
        parser.add_argument('--branch', '-b', help='Git branch to deploy from', default='master')
        parser.add_argument('--profile', help='Harbor profile to filter out services that needs to be deployed',
                            default='')
        self._add_ask_pass_arguments_to_argparse(parser)
        self._add_vault_arguments_to_argparse(parser)

    def run(self, context: ExecutionContext) -> bool:
        playbook_name = context.get_arg_or_env('--playbook')
        inventory_name = context.get_arg_or_env('--inventory')
        git_private_key_path = context.get_arg_or_env('--git-key')
        branch = context.get_arg('--branch')
        profile = context.get_arg('--profile')
        debug = context.get_arg('--debug')

        # keep the vault arguments for decryption of deployment.yml
        self._preserve_vault_parameters_for_usage_in_inner_tasks(context)

        if not self.role_is_installed_and_configured():
            self.io().error_msg('Deployment not configured. Use `harbor :deployment:files:update` first')
            return False

        try:
            self.install_and_configure_role(context, force_update=False)

        except MissingDeploymentConfigurationError as e:
            self.io().error_msg(str(e))
            return False

        pwd_backup = os.getcwd()
        pid = None

        try:
            command = ''
            opts = ''

            if git_private_key_path:
                sock, pid = self.spawn_ssh_agent()
                command += 'export SSH_AUTH_SOCK=%s; export SSH_AGENT_PID=%i; ssh-add %s; sleep 5; ' % \
                           (sock, pid, git_private_key_path)

            if debug:
                opts += ' -vv '

            opts += ' -e git_branch="%s" ' % branch
            opts += ' -e harbor_deployment_profile="%s" ' % profile
            opts += self._get_vault_opts(context, '../../')

            os.chdir(self.ansible_dir)
            command += 'ansible-playbook ./%s -i %s %s' % (
                playbook_name,
                inventory_name,
                opts
            )

            self.spawn_ansible(command)
        finally:
            os.chdir(pwd_backup)

            if pid:
                self.kill_ssh_agent(pid)

        return True

    def spawn_ansible(self, command):
        self.io().info('Spawning Ansible, you may be asked for vault password to decrypt .env-prod')
        return self.sh(command)

    def spawn_ssh_agent(self) -> Tuple[str, int]:
        out = subprocess.check_output('eval $(ssh-agent -s);echo "|${SSH_AUTH_SOCK}|${SSH_AGENT_PID}";', shell=True).decode('utf-8')
        parts = out.split('|')
        sock = parts[1]
        pid = int(parts[2].strip())

        self.io().debug('Spawned ssh-agent - sock=%s, pid=%i' % (sock, pid))

        return sock, pid

    def kill_ssh_agent(self, pid: int):
        self.io().debug('Clean up - killing ssh-agent at PID=%i' % pid)
        subprocess.check_call(['kill', str(pid)])


class CreateExampleDeploymentFileTask(HarborBaseTask):
    """Creates a example deployment.yml file"""

    def get_group_name(self) -> str:
        return ':harbor:deployment'

    def get_name(self) -> str:
        return ':create-example'

    def format_task_name(self, name) -> str:
        return development_formatting(name)

    def run(self, context: ExecutionContext) -> bool:
        if os.path.isfile('./deployment.yml') or os.path.isfile('./deployment.yaml'):
            self.io().error_msg('deployment.yml or deployment.yaml already exists')
            return False

        subprocess.check_call(['cp', HARBOR_ROOT + '/../examples/deployment.yml', './deployment.yml'])
        self.io().success_msg('File "deployment.yml" created.')
        self.io().print_line()
        self.io().info_msg('The example is initially adjusted to work with Vagrant test virtual machine.')
        self.io().info_msg(' - `harbor :deployment:vagrant -c "up --provision"` to bring machine up')
        self.io().info_msg(' - `harbor :deployment:apply --git-key=~/.ssh/id_rsa` to perform a test deployment')

        return True




from typing import Dict
from typing import Optional
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from ...formatting import development_formatting
from ...exception import MissingDeploymentConfigurationError
from .base import BaseDeploymentTask


class SSHTask(BaseDeploymentTask):
    """SSH into a host defined in a deployment configuration"""

    def get_name(self) -> str:
        return ':ssh'

    def get_group_name(self) -> str:
        return ':harbor:deployment'

    def format_task_name(self, name) -> str:
        return development_formatting(name)

    def configure_argparse(self, parser: ArgumentParser):
        self._add_ask_pass_arguments_to_argparse(parser)
        self._add_vault_arguments_to_argparse(parser)

        parser.add_argument('--group', help='Node group eg. "production"', required=True)
        parser.add_argument('--num', help='Node number from given group, defaults to "1"', default="1")
        parser.add_argument('--print-password', help='Print user password', action='store_true')

    def get_declared_envs(self) -> Dict[str, str]:
        envs = super().get_declared_envs()
        envs['VAULT_PASSWORDS'] = ''

        return envs

    def run(self, context: ExecutionContext) -> bool:
        self._preserve_vault_parameters_for_usage_in_inner_tasks(context)

        node_group = context.get_arg('--group')
        node_num = int(context.get_arg('--num')) - 1
        should_print_password = context.get_arg('--print-password')

        try:
            config = self.get_config()
            node = self._get_node(node_group, node_num, config)

            if not node:
                return False

            if should_print_password:
                self._print_password(node)

            return self._ssh(node)

        except MissingDeploymentConfigurationError as e:
            self.io().error_msg(str(e))
            return False

    def _get_node(self, node_group: str, node_num: int, config: dict) -> Optional[dict]:
        if node_group not in config['nodes'].keys():
            self.io().error_msg('Node group "{}" not found'.format(node_group))
            return None

        try:
            return config['nodes'][node_group][node_num]
        except IndexError:
            self.io().error_msg('Node group "{}" does not have node of number #{}'.format(node_group, node_num + 1))
            return None

    def _print_password(self, node: dict):
        if 'sudo_password' in node:
            self.io().info_msg('>> SUDO password is: "{}"'.format(node['sudo_password']))
            return

        if 'password' in node:
            self.io().info_msg('>> User password is: "{}"'.format(node['password']))
            return

    def _ssh(self, node: dict) -> bool:
        ssh_cmd = 'ssh {}@{} -p {}'.format(node['user'], node['host'], node['port'])

        has_private_key = 'private_key' in node
        has_password = 'password' in node

        if has_private_key:
            ssh_cmd += ' -i {}'.format(node['private_key'])

        if not has_private_key and has_password:
            ssh_cmd = 'sshpass -p "{}" '.format(node['password']) + ssh_cmd

        self.sh(ssh_cmd, use_subprocess=True)

        return True



from typing import Tuple
from typing import List
from rkd_harbor.test import BaseHarborTestClass
from rkd_harbor.tasks.deployment.ssh import SSHTask


class SSHTaskTest(BaseHarborTestClass):
    def _execute_task_and_get_sh_calls_and_output(self, args: dict, mock_exceptions: list = None) -> Tuple[List[str], str]:
        calls = []
        task = SSHTask()
        original_sh = task.sh

        def mocked_sh(*args, **kwargs):
            if mock_exceptions:
                for exc in mock_exceptions:
                    if exc in args[0]:
                        return original_sh(*args, **kwargs)

            calls.append(args[0])

        task.sh = mocked_sh

        out = self.execute_mocked_task_and_get_output(
            task,
            args=args,
            env={}
        )

        return calls, out

    def test_ssh_task_creates_a_valid_ssh_command_without_private_key(self):
        """
        Test that ssh command would be invoked along with sshpass - as there is no private key
        """

        self.prepare_valid_deployment_yml()

        sh_calls, out = self._execute_task_and_get_sh_calls_and_output({
            '--ask-vault-pass': False,
            '--vault-passwords': '',
            '--ask-ssh-login': False,
            '--ask-ssh-pass': False,
            '--ask-ssh-key-path': False,
            '--ask-sudo-pass': False,
            '--group': 'production',
            '--num': '1',
            '--print-password': False
        })

        self.assertEqual(['sshpass -p "docker" ssh docker@127.0.0.1 -p 2222'], sh_calls)

    def test_ssh_task_includes_private_key_if_present(self):
        """
        Test that private key, encoded inline will be correctly handled
        """

        self.prepare_valid_deployment_yml_with_private_key()

        sh_calls, out = self._execute_task_and_get_sh_calls_and_output({
            '--ask-vault-pass': False,
            '--vault-passwords': '',
            '--ask-ssh-login': False,
            '--ask-ssh-pass': False,
            '--ask-ssh-key-path': False,
            '--ask-sudo-pass': False,
            '--group': 'production',
            '--num': '1',
            '--print-password': False
        })

        self.assertIn('ssh docker@127.0.0.1 -p 2222 -i /', str(sh_calls))
        self.assertIn('current_test_env/.rkd/.tmp-', str(sh_calls))

    def test_ssh_prints_password_if_sudo_password_is_given(self):
        """
        Test that SUDO password will be print if specified

        See prepare_valid_deployment_yml_with_private_key() for data example
        """

        self.prepare_valid_deployment_yml_with_private_key()

        sh_calls, out = self._execute_task_and_get_sh_calls_and_output({
            '--ask-vault-pass': False,
            '--vault-passwords': '',
            '--ask-ssh-login': False,
            '--ask-ssh-pass': False,
            '--ask-ssh-key-path': False,
            '--ask-sudo-pass': False,
            '--group': 'production',
            '--num': '1',
            '--print-password': True
        })

        self.assertIn('SUDO password is: "sudo-docker"', out)

    def test_ssh_prints_user_password_if_sudo_not_present(self):
        """
        Test that SSH user password will be printed if "sudo_password" not present

        See prepare_valid_deployment_yml() for details
        """

        self.prepare_valid_deployment_yml()

        sh_calls, out = self._execute_task_and_get_sh_calls_and_output({
            '--ask-vault-pass': False,
            '--vault-passwords': '',
            '--ask-ssh-login': False,
            '--ask-ssh-pass': False,
            '--ask-ssh-key-path': False,
            '--ask-sudo-pass': False,
            '--group': 'production',
            '--num': '1',
            '--print-password': True
        })

        self.assertIn('User password is: "docker"', out)
        
    def test_ssh_num_too_high_will_be_checked(self):
        """
        Test that if we specify eg. server #2, but we have only one server, then a proper error will be shown
        """

        self.prepare_valid_deployment_yml()

        sh_calls, out = self._execute_task_and_get_sh_calls_and_output({
            '--ask-vault-pass': False,
            '--vault-passwords': '',
            '--ask-ssh-login': False,
            '--ask-ssh-pass': False,
            '--ask-ssh-key-path': False,
            '--ask-sudo-pass': False,
            '--group': 'production',
            '--num': '2',
            '--print-password': True
        })

        self.assertIn('Node group "production" does not have node of number #2', out)

    def test_ssh_decrypts_yaml_when_encoded(self):
        """
        Test that SSH user password will be printed if "sudo_password" not present

        See prepare_valid_deployment_yml() for details
        """

        # do not mock specific sh() calls, so the workflow could be preserved
        mock_exceptions = ['cp deployment.yml', ':harbor:vault:encrypt']

        self.prepare_valid_deployment_yml_encrypted_with_password_test()

        sh_calls, out = self._execute_task_and_get_sh_calls_and_output({
            '--ask-vault-pass': False,
            '--vault-passwords': 'test',
            '--ask-ssh-login': False,
            '--ask-ssh-pass': False,
            '--ask-ssh-key-path': False,
            '--ask-sudo-pass': False,
            '--group': 'production',
            '--num': '1',
            '--print-password': True
        }, mock_exceptions=mock_exceptions)

        self.assertIn('User password is: "docker"', out)



from .apply import DeploymentTask
from .apply import CreateExampleDeploymentFileTask
from .syncfiles import UpdateFilesTask
from .vagrant import ManageVagrantTask
from .vault import EncryptVaultTask
from .vault import EnvEncryptTask
from .vault import EditVaultTask
from .ssh import SSHTask


def imports() -> list:
    return [
        DeploymentTask(),
        SSHTask(),
        CreateExampleDeploymentFileTask(),
        UpdateFilesTask(),
        ManageVagrantTask(),
        EncryptVaultTask(),
        EnvEncryptTask(),
        EditVaultTask()
    ]



import subprocess
from typing import Dict
from argparse import ArgumentParser
from rkd.api.contract import ExecutionContext
from ...formatting import development_formatting
from .base import BaseDeploymentTask


class EditVaultTask(BaseDeploymentTask):
    """Edits an encrypted file

Example usage:
    # edit ".env-prod" file
    harbor :vault:edit .env-prod --vault-passwords="./.vault-password"

    # usage of environment variable (NOTICE: paths to password files must begin with "/" or "./")
    VAULT_PASSWORDS="./.vault-password-file||second-some-plaintext-password-there" harbor :vault:edit .env-prod

    # use a different text editor (you can also put EDITOR variable to your .env file)
    EDITOR=vim harbor :vault:edit deployment.yml

HINT: You can avoid writing the path in commandline each time by putting `VAULT_PASSWORDS=./path-to-password-file.txt` to the .env file
HINT: You can store vault password file on encrypted flash drive, and make a symbolic link. Every time when you mount an encrypted drive you will gain access to the project
NOTICE: When at least one of vault password files does not exist, then there will be a password prompt
"""

    def get_group_name(self) -> str:
        return ':harbor:vault'

    def get_name(self) -> str:
        return ':edit'

    def get_declared_envs(self) -> Dict[str, str]:
        envs = super(BaseDeploymentTask, self).get_declared_envs()
        envs['VAULT_PASSWORDS'] = ''

        return envs

    def configure_argparse(self, parser: ArgumentParser):
        parser.add_argument('filename', help='Filename')
        self._add_vault_arguments_to_argparse(parser)

    def run(self, context: ExecutionContext) -> bool:
        vault_opts = self._get_vault_opts(context)
        filename = context.get_arg('filename')

        subprocess.check_call('ansible-vault edit %s %s' % (vault_opts, filename), shell=True)

        return True


class EncryptVaultTask(BaseDeploymentTask):
    """Encrypts/Decrypts a file using strong AES-256 algorithm,
output files are suitable to be kept in GIT repository

See the documentation for :harbor:vault:edit task for general file encryption documentation
"""

    def get_group_name(self) -> str:
        return ':harbor:vault'

    def get_name(self) -> str:
        return ':encrypt'

    def get_declared_envs(self) -> Dict[str, str]:
        envs = super(BaseDeploymentTask, self).get_declared_envs()
        envs['VAULT_PASSWORDS'] = ''

        return envs

    def configure_argparse(self, parser: ArgumentParser):
        parser.add_argument('--decrypt', '-d', action='store_true', help='Decrypt instead of encrypting')
        parser.add_argument('filename', help='Filename')
        self._add_vault_arguments_to_argparse(parser)

    def run(self, context: ExecutionContext) -> bool:
        vault_opts = self._get_vault_opts(context)
        filename = context.get_arg('filename')
        mode = 'decrypt' if context.get_arg('--decrypt') else 'encrypt'

        self.sh('ansible-vault %s %s %s' % (mode, vault_opts, filename), capture=False)

        return True


class EnvEncryptTask(BaseDeploymentTask):
    """Manages the encryption of .env-prod file

The .env-prod file is a file that could be kept securely in GIT repository while containing passwords
required for services to work.
"""

    def get_group_name(self) -> str:
        return ':harbor:env'

    def get_name(self) -> str:
        return ':encrypt'

    def format_task_name(self, name) -> str:
        return development_formatting(name)

    def get_declared_envs(self) -> Dict[str, str]:
        envs = super(BaseDeploymentTask, self).get_declared_envs()
        envs['VAULT_PASSWORDS'] = ''

        return envs

    def configure_argparse(self, parser: ArgumentParser):
        parser.add_argument('--decrypt', '-d', action='store_true', help='Decrypt instead of encrypting')
        self._add_vault_arguments_to_argparse(parser)

    def run(self, context: ExecutionContext) -> bool:
        vault_opts = self._get_vault_opts(context)
        mode = 'decrypt' if context.get_arg('--decrypt') else 'encrypt'

        src = '.env'
        dst = '.env-prod'

        if mode == 'decrypt':
            src = '.env-prod'
            dst = '.env'

        self.sh('cp %s %s-tmp' % (src, dst))
        self.sh('ansible-vault %s %s %s-tmp' % (mode, vault_opts, dst), capture=False)
        self.sh('mv %s-tmp %s' % (dst, dst))

        if mode == 'encrypt':
            try:
                self.sh('git add %s' % dst)
            except:
                pass

        return True



import os
import tempfile
from rkd_harbor.test import BaseHarborTestClass
from rkd_harbor.tasks.structure import CreateHarborStructureTask
from rkd.api.contract import ExecutionContext
from rkd.api.inputoutput import IO
from rkd.test import get_test_declaration


class CreateHarborStructureTaskTest(BaseHarborTestClass):

    def test_functional_installation(self):
        """Test functionally that the structure will be created properly

        NOTICE: This test has external dependencies such as PyPI
        """

        backup_dir_path = os.getcwd()

        try:
            with tempfile.TemporaryDirectory() as dir_path:
                os.chdir(dir_path)

                task = CreateHarborStructureTask()

                # do not write requirements.txt in tests, we do not want to install all of those things
                # as it is very problematic in development environment where things are "under construction"
                task.on_requirements_txt_write = lambda ctx: None

                self.execute_mocked_task_and_get_output(task, args={
                    '--commit': False,
                    '--no-venv': False,
                    '--pipenv': False
                })

                self.assertTrue(os.path.isfile(dir_path + '/.env-default'),
                                msg='Expected that dot files will be copied')

                self.assertTrue(os.path.isfile(dir_path + '/.venv/bin/activate'),
                                msg='Expected the virtualenv to be installed')

                self.assertTrue(os.path.isfile(dir_path + '/apps/README.md'),
                                msg='Expected that files in inner directories will be present')

        finally:
            os.chdir(backup_dir_path)

    def test_requirements_are_written(self):
        """Verify that basic requirements list is written into requirements.txt file"""

        backup_dir_path = os.getcwd()

        try:
            with tempfile.TemporaryDirectory() as dir_path:
                os.chdir(dir_path)

                task = CreateHarborStructureTask()
                task._io = IO()
                task.on_requirements_txt_write(ExecutionContext(get_test_declaration()))

                with open(dir_path + '/requirements.txt', 'rb') as requirements_txt:
                    content = requirements_txt.read().decode('utf-8')

                    self.assertIn('ansible>=2.8', content, msg='Expected Ansible defined in requirements.txt')
                    self.assertIn('rkd-harbor==', content, msg='Expected rkd-harbor to be defined in requirements.txt')

        finally:
            os.chdir(backup_dir_path)
